@@ -121,12 +121,13 @@ Op_Any_RGB_to_YCbCr_420_Sharp::state_after_conversion(
 #endif
 }
 
-std::shared_ptr<HeifPixelImage>
+Result<std::shared_ptr<HeifPixelImage>>
 Op_Any_RGB_to_YCbCr_420_Sharp::convert_colorspace(
     const std::shared_ptr<const HeifPixelImage>& input,
     const ColorState& input_state,
     const ColorState& target_state,
-    const heif_color_conversion_options& options) const
+    const heif_color_conversion_options& options,
+    const heif_security_limits* limits) const
 {
 #ifdef HAVE_LIBSHARPYUV
   uint32_t width = input->get_width();
@@ -154,15 +155,15 @@ Op_Any_RGB_to_YCbCr_420_Sharp::convert_colorspace(
   bool want_alpha = target_state.has_alpha;
 
   int output_bits = target_state.bits_per_pixel;
-  if (!outimg->add_plane(heif_channel_Y, width, height, output_bits) ||
-      !outimg->add_plane(heif_channel_Cb, chroma_width, chroma_height, output_bits) ||
-      !outimg->add_plane(heif_channel_Cr, chroma_width, chroma_height, output_bits)) {
-    return nullptr;
+  if (auto err = outimg->add_plane(heif_channel_Y, width, height, output_bits, limits) ||
+                 outimg->add_plane(heif_channel_Cb, chroma_width, chroma_height, output_bits, limits) ||
+                 outimg->add_plane(heif_channel_Cr, chroma_width, chroma_height, output_bits, limits)) {
+    return err;
   }
 
   if (want_alpha) {
-    if (!outimg->add_plane(heif_channel_Alpha, width, height, output_bits)) {
-      return nullptr;
+    if (auto err = outimg->add_plane(heif_channel_Alpha, width, height, output_bits, limits)) {
+      return err;
     }
   }
 
@@ -186,14 +187,14 @@ Op_Any_RGB_to_YCbCr_420_Sharp::convert_colorspace(
     in_b = input->get_plane(heif_channel_B, &in_b_stride);
     // The stride must be the same for all channels.
     if (in_r_stride != in_g_stride || in_r_stride != in_b_stride) {
-      return nullptr;
+      return Error::InternalError;
     }
     in_stride = in_r_stride;
     // Bpp must also be the same.
     input_bits = input->get_bits_per_pixel(heif_channel_R);
     if (input_bits != input->get_bits_per_pixel(heif_channel_G) ||
         input_bits != input->get_bits_per_pixel(heif_channel_B)) {
-      return nullptr;
+      return Error::InternalError;
     }
     if (has_alpha) {
       in_a = input->get_plane(heif_channel_Alpha, &in_a_stride);
@@ -237,7 +238,9 @@ Op_Any_RGB_to_YCbCr_420_Sharp::convert_colorspace(
                       out_cr, out_cr_stride, output_bits,
                       input->get_width(), input->get_height(), &yuv_matrix);
   if (!sharpyuv_ok) {
-    return nullptr;
+    return Error{heif_error_Unsupported_feature,
+                 heif_suberror_Unsupported_color_conversion,
+                 "SharpYuv color convertion failed"};
   }
 
   if (want_alpha) {
@@ -271,6 +274,6 @@ Op_Any_RGB_to_YCbCr_420_Sharp::convert_colorspace(
 
   return outimg;
 #else
-  return nullptr;
+  return Error::InternalError;
 #endif
 }
